// This file is generated by kconfig_compiler from kcm_kdnssd.kcfg.
// All changes you do to this file will be lost.

#include "settings.h"

#include <kglobal.h>
#include <QtCore/QFile>

using namespace DNSSD;

namespace DNSSD {

class ConfigurationHelper
{
  public:
    ConfigurationHelper() : q(0) {}
    ~ConfigurationHelper() { delete q; }
    Configuration *q;
};
}

K_GLOBAL_STATIC(ConfigurationHelper, s_globalConfiguration)
Configuration *Configuration::self()
{
  if (!s_globalConfiguration->q) {
    new Configuration;
    s_globalConfiguration->q->readConfig();
  }

  return s_globalConfiguration->q;
}

Configuration::Configuration(  )
  : KConfigSkeleton( QLatin1String( "kdnssdrc" ) )
{
  Q_ASSERT(!s_globalConfiguration->q);
  s_globalConfiguration->q = this;
  setCurrentGroup( QLatin1String( "browsing" ) );

  KConfigSkeleton::ItemStringList  *itemDomainList;
  itemDomainList = new KConfigSkeleton::ItemStringList( currentGroup(), QLatin1String( "DomainList" ), mDomainList );
  addItem( itemDomainList, QLatin1String( "DomainList" ) );
}

Configuration::~Configuration()
{
  if (!s_globalConfiguration.isDestroyed()) {
    s_globalConfiguration->q = 0;
  }
}

