/****************************************************************************
** Meta object code from reading C++ file 'khtmlfindbar.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.7)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/kdelibs-4.14.38/khtml/ui/findbar/khtmlfindbar.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'khtmlfindbar.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.7. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_KHTMLFindBar[] = {

 // content:
       6,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: signature, parameters, type, tag, flags
      14,   13,   13,   13, 0x05,
      30,   13,   13,   13, 0x05,
      48,   13,   13,   13, 0x05,

 // slots: signature, parameters, type, tag, flags
      76,   70,   13,   13, 0x08,
     106,   13,   13,   13, 0x08,
     126,   13,   13,   13, 0x08,

       0        // eod
};

static const char qt_meta_stringdata_KHTMLFindBar[] = {
    "KHTMLFindBar\0\0searchChanged()\0"
    "findNextClicked()\0findPreviousClicked()\0"
    "selec\0slotSelectedTextToggled(bool)\0"
    "slotSearchChanged()\0slotAddPatternToHistory()\0"
};

void KHTMLFindBar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        KHTMLFindBar *_t = static_cast<KHTMLFindBar *>(_o);
        switch (_id) {
        case 0: _t->searchChanged(); break;
        case 1: _t->findNextClicked(); break;
        case 2: _t->findPreviousClicked(); break;
        case 3: _t->slotSelectedTextToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->slotSearchChanged(); break;
        case 5: _t->slotAddPatternToHistory(); break;
        default: ;
        }
    }
}

const QMetaObjectExtraData KHTMLFindBar::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject KHTMLFindBar::staticMetaObject = {
    { &KHTMLViewBarWidget::staticMetaObject, qt_meta_stringdata_KHTMLFindBar,
      qt_meta_data_KHTMLFindBar, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &KHTMLFindBar::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *KHTMLFindBar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *KHTMLFindBar::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_KHTMLFindBar))
        return static_cast<void*>(const_cast< KHTMLFindBar*>(this));
    return KHTMLViewBarWidget::qt_metacast(_clname);
}

int KHTMLFindBar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KHTMLViewBarWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void KHTMLFindBar::searchChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, 0);
}

// SIGNAL 1
void KHTMLFindBar::findNextClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 1, 0);
}

// SIGNAL 2
void KHTMLFindBar::findPreviousClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 2, 0);
}
QT_END_MOC_NAMESPACE
