/****************************************************************************
** Resource object code
**
** Created by: The Resource Compiler for Qt version 4.8.7
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtCore/qglobal.h>

QT_BEGIN_NAMESPACE

QT_END_NAMESPACE


int QT_MANGLE_NAMESPACE(qInitResources_kdewebkitwidgets)()
{
    return 1;
}

Q_CONSTRUCTOR_FUNCTION(QT_MANGLE_NAMESPACE(qInitResources_kdewebkitwidgets))

int QT_MANGLE_NAMESPACE(qCleanupResources_kdewebkitwidgets)()
{
    return 1;
}

Q_DESTRUCTOR_FUNCTION(QT_MANGLE_NAMESPACE(qCleanupResources_kdewebkitwidgets))

