#include <kdialog.h>
#include <klocale.h>

/*
Licensed under GNU LGPL
*/

/********************************************************************************
** Form generated from reading UI file 'configui.ui'
**
** Created by: Qt User Interface Compiler version 4.8.7
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIGUI_H
#define UI_CONFIGUI_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QCheckBox>
#include <QtGui/QGridLayout>
#include <QtGui/QGroupBox>
#include <QtGui/QHeaderView>
#include <QtGui/QLabel>
#include <QtGui/QVBoxLayout>
#include <QtGui/QWidget>
#include "kcombobox.h"
#include "keditlistwidget.h"
#include "sonnet/dictionarycombobox.h"

QT_BEGIN_NAMESPACE

class Ui_SonnetConfigUI
{
public:
    QGridLayout *gridLayout;
    Sonnet::DictionaryComboBox *m_langCombo;
    QGroupBox *groupBox1;
    QGridLayout *gridLayout1;
    QCheckBox *m_bgSpellCB;
    QCheckBox *m_checkerEnabledByDefaultCB;
    QCheckBox *m_skipUpperCB;
    QCheckBox *m_skipRunTogetherCB;
    QLabel *textLabel1;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout;
    KEditListWidget *m_ignoreListBox;

    void setupUi(QWidget *SonnetConfigUI)
    {
        if (SonnetConfigUI->objectName().isEmpty())
            SonnetConfigUI->setObjectName(QString::fromUtf8("SonnetConfigUI"));
        SonnetConfigUI->resize(577, 441);
        gridLayout = new QGridLayout(SonnetConfigUI);
        gridLayout->setContentsMargins(0, 0, 0, 0);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        m_langCombo = new Sonnet::DictionaryComboBox(SonnetConfigUI);
        m_langCombo->setObjectName(QString::fromUtf8("m_langCombo"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(m_langCombo->sizePolicy().hasHeightForWidth());
        m_langCombo->setSizePolicy(sizePolicy);

        gridLayout->addWidget(m_langCombo, 0, 1, 1, 1);

        groupBox1 = new QGroupBox(SonnetConfigUI);
        groupBox1->setObjectName(QString::fromUtf8("groupBox1"));
        gridLayout1 = new QGridLayout(groupBox1);
        gridLayout1->setObjectName(QString::fromUtf8("gridLayout1"));
        m_bgSpellCB = new QCheckBox(groupBox1);
        m_bgSpellCB->setObjectName(QString::fromUtf8("m_bgSpellCB"));

        gridLayout1->addWidget(m_bgSpellCB, 0, 0, 1, 1);

        m_checkerEnabledByDefaultCB = new QCheckBox(groupBox1);
        m_checkerEnabledByDefaultCB->setObjectName(QString::fromUtf8("m_checkerEnabledByDefaultCB"));

        gridLayout1->addWidget(m_checkerEnabledByDefaultCB, 1, 0, 1, 1);

        m_skipUpperCB = new QCheckBox(groupBox1);
        m_skipUpperCB->setObjectName(QString::fromUtf8("m_skipUpperCB"));

        gridLayout1->addWidget(m_skipUpperCB, 2, 0, 1, 1);

        m_skipRunTogetherCB = new QCheckBox(groupBox1);
        m_skipRunTogetherCB->setObjectName(QString::fromUtf8("m_skipRunTogetherCB"));

        gridLayout1->addWidget(m_skipRunTogetherCB, 3, 0, 1, 1);


        gridLayout->addWidget(groupBox1, 1, 0, 1, 2);

        textLabel1 = new QLabel(SonnetConfigUI);
        textLabel1->setObjectName(QString::fromUtf8("textLabel1"));
        textLabel1->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(textLabel1, 0, 0, 1, 1);

        groupBox = new QGroupBox(SonnetConfigUI);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        verticalLayout = new QVBoxLayout(groupBox);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        m_ignoreListBox = new KEditListWidget(groupBox);
        m_ignoreListBox->setObjectName(QString::fromUtf8("m_ignoreListBox"));

        verticalLayout->addWidget(m_ignoreListBox);


        gridLayout->addWidget(groupBox, 2, 0, 1, 2);


        retranslateUi(SonnetConfigUI);

        QMetaObject::connectSlotsByName(SonnetConfigUI);
    } // setupUi

    void retranslateUi(QWidget *SonnetConfigUI)
    {
        groupBox1->setTitle(tr2i18n("Options", 0));
        m_bgSpellCB->setText(tr2i18n("Enable &background spellchecking", 0));
        m_checkerEnabledByDefaultCB->setText(tr2i18n("&Automatic spell checking enabled by default", 0));
        m_skipUpperCB->setText(tr2i18n("Skip all &uppercase words", 0));
        m_skipRunTogetherCB->setText(tr2i18n("S&kip run-together words", 0));
        textLabel1->setText(tr2i18n("Default language:", 0));
        groupBox->setTitle(tr2i18n("Ignored Words", 0));
        m_ignoreListBox->setProperty("title", QVariant(tr2i18n("Ignored Words", 0)));
        Q_UNUSED(SonnetConfigUI);
    } // retranslateUi

};

namespace Ui {
    class SonnetConfigUI: public Ui_SonnetConfigUI {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CONFIGUI_H

