#include <kdialog.h>
#include <klocale.h>

/********************************************************************************
** Form generated from reading UI file 'kcupsoptionsjobwidget.ui'
**
** Created by: Qt User Interface Compiler version 4.8.7
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KCUPSOPTIONSJOBWIDGET_H
#define UI_KCUPSOPTIONSJOBWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QGridLayout>
#include <QtGui/QGroupBox>
#include <QtGui/QHeaderView>
#include <QtGui/QLabel>
#include <QtGui/QSpacerItem>
#include <QtGui/QTableWidget>
#include <QtGui/QTimeEdit>
#include <QtGui/QWidget>
#include "kcombobox.h"
#include "klineedit.h"
#include "knuminput.h"

QT_BEGIN_NAMESPACE

class Ui_KCupsOptionsJobWidget
{
public:
    QGridLayout *gridLayout_3;
    QGroupBox *jobControlGroupBox;
    QGridLayout *gridLayout;
    QLabel *jobHoldLabel;
    KComboBox *jobHoldComboBox;
    QTimeEdit *jobHoldTimeEdit;
    QLabel *jobBillingLabel;
    KLineEdit *jobBillingLineEdit;
    QLabel *jobPriorityLabel;
    KIntSpinBox *jobPrioritySpinBox;
    QSpacerItem *horizontalSpacer;
    QSpacerItem *verticalSpacer;
    QGroupBox *jobOptionsGroupBox;
    QGridLayout *gridLayout_2;
    QTableWidget *jobOptionsTable;
    QSpacerItem *verticalSpacer_2;
    QSpacerItem *verticalSpacer_3;

    void setupUi(QWidget *KCupsOptionsJobWidget)
    {
        if (KCupsOptionsJobWidget->objectName().isEmpty())
            KCupsOptionsJobWidget->setObjectName(QString::fromUtf8("KCupsOptionsJobWidget"));
        KCupsOptionsJobWidget->resize(400, 294);
        gridLayout_3 = new QGridLayout(KCupsOptionsJobWidget);
        gridLayout_3->setObjectName(QString::fromUtf8("gridLayout_3"));
        jobControlGroupBox = new QGroupBox(KCupsOptionsJobWidget);
        jobControlGroupBox->setObjectName(QString::fromUtf8("jobControlGroupBox"));
        gridLayout = new QGridLayout(jobControlGroupBox);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        jobHoldLabel = new QLabel(jobControlGroupBox);
        jobHoldLabel->setObjectName(QString::fromUtf8("jobHoldLabel"));

        gridLayout->addWidget(jobHoldLabel, 0, 0, 1, 1);

        jobHoldComboBox = new KComboBox(jobControlGroupBox);
        jobHoldComboBox->setObjectName(QString::fromUtf8("jobHoldComboBox"));

        gridLayout->addWidget(jobHoldComboBox, 0, 1, 1, 2);

        jobHoldTimeEdit = new QTimeEdit(jobControlGroupBox);
        jobHoldTimeEdit->setObjectName(QString::fromUtf8("jobHoldTimeEdit"));

        gridLayout->addWidget(jobHoldTimeEdit, 0, 3, 1, 1);

        jobBillingLabel = new QLabel(jobControlGroupBox);
        jobBillingLabel->setObjectName(QString::fromUtf8("jobBillingLabel"));

        gridLayout->addWidget(jobBillingLabel, 1, 0, 1, 1);

        jobBillingLineEdit = new KLineEdit(jobControlGroupBox);
        jobBillingLineEdit->setObjectName(QString::fromUtf8("jobBillingLineEdit"));

        gridLayout->addWidget(jobBillingLineEdit, 1, 1, 1, 3);

        jobPriorityLabel = new QLabel(jobControlGroupBox);
        jobPriorityLabel->setObjectName(QString::fromUtf8("jobPriorityLabel"));

        gridLayout->addWidget(jobPriorityLabel, 2, 0, 1, 1);

        jobPrioritySpinBox = new KIntSpinBox(jobControlGroupBox);
        jobPrioritySpinBox->setObjectName(QString::fromUtf8("jobPrioritySpinBox"));
        jobPrioritySpinBox->setMaximum(100);
        jobPrioritySpinBox->setValue(50);

        gridLayout->addWidget(jobPrioritySpinBox, 2, 1, 1, 1);

        horizontalSpacer = new QSpacerItem(180, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer, 2, 2, 1, 2);

        verticalSpacer = new QSpacerItem(20, 35, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout->addItem(verticalSpacer, 3, 1, 1, 1);


        gridLayout_3->addWidget(jobControlGroupBox, 0, 0, 1, 1);

        jobOptionsGroupBox = new QGroupBox(KCupsOptionsJobWidget);
        jobOptionsGroupBox->setObjectName(QString::fromUtf8("jobOptionsGroupBox"));
        gridLayout_2 = new QGridLayout(jobOptionsGroupBox);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        jobOptionsTable = new QTableWidget(jobOptionsGroupBox);
        if (jobOptionsTable->columnCount() < 2)
            jobOptionsTable->setColumnCount(2);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        jobOptionsTable->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        jobOptionsTable->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        if (jobOptionsTable->rowCount() < 10)
            jobOptionsTable->setRowCount(10);
        jobOptionsTable->setObjectName(QString::fromUtf8("jobOptionsTable"));
        jobOptionsTable->setRowCount(10);
        jobOptionsTable->setColumnCount(2);

        gridLayout_2->addWidget(jobOptionsTable, 0, 0, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 16, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout_2->addItem(verticalSpacer_2, 1, 0, 1, 1);


        gridLayout_3->addWidget(jobOptionsGroupBox, 1, 0, 1, 1);

        verticalSpacer_3 = new QSpacerItem(20, 13, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout_3->addItem(verticalSpacer_3, 2, 0, 1, 1);


        retranslateUi(KCupsOptionsJobWidget);

        QMetaObject::connectSlotsByName(KCupsOptionsJobWidget);
    } // setupUi

    void retranslateUi(QWidget *KCupsOptionsJobWidget)
    {
        KCupsOptionsJobWidget->setWindowTitle(tr2i18n("Job", 0));
        jobControlGroupBox->setTitle(tr2i18n("Job Control", 0));
        jobHoldLabel->setText(tr2i18n("Scheduled printing:", 0));
        jobBillingLabel->setText(tr2i18n("Billing information:", 0));
        jobPriorityLabel->setText(tr2i18n("Job priority:", 0));
        jobOptionsGroupBox->setTitle(tr2i18n("Job Options", 0));
        QTableWidgetItem *___qtablewidgetitem = jobOptionsTable->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(tr2i18n("Option", 0));
        QTableWidgetItem *___qtablewidgetitem1 = jobOptionsTable->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(tr2i18n("Value", 0));
    } // retranslateUi

};

namespace Ui {
    class KCupsOptionsJobWidget: public Ui_KCupsOptionsJobWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KCUPSOPTIONSJOBWIDGET_H

