/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp -m -i kcookiejar_include.h -p kcookieserverinterface /usr/src/ports/kdelibs/kdelibs-4.14.38-2.x86_64/build/kioslave/http/kcookiejar/org.kde.KCookieServer.xml
 *
 * qdbusxml2cpp is Copyright (C) 2015 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef KCOOKIESERVERINTERFACE_H_1514339316
#define KCOOKIESERVERINTERFACE_H_1514339316

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>
#include "kcookiejar_include.h"

/*
 * Proxy class for interface org.kde.KCookieServer
 */
class OrgKdeKCookieServerInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.kde.KCookieServer"; }

public:
    OrgKdeKCookieServerInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = 0);

    ~OrgKdeKCookieServerInterface();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> addCookies(const QString &url, const QByteArray &cookieHeader, qlonglong windowId)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(url) << QVariant::fromValue(cookieHeader) << QVariant::fromValue(windowId);
        return asyncCallWithArgumentList(QLatin1String("addCookies"), argumentList);
    }

    inline QDBusPendingReply<> addDOMCookies(const QString &url, const QByteArray &cookieHeader, qlonglong windowId)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(url) << QVariant::fromValue(cookieHeader) << QVariant::fromValue(windowId);
        return asyncCallWithArgumentList(QLatin1String("addDOMCookies"), argumentList);
    }

    inline QDBusPendingReply<> deleteAllCookies()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QLatin1String("deleteAllCookies"), argumentList);
    }

    inline QDBusPendingReply<> deleteCookie(const QString &domain, const QString &fqdn, const QString &path, const QString &name)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(domain) << QVariant::fromValue(fqdn) << QVariant::fromValue(path) << QVariant::fromValue(name);
        return asyncCallWithArgumentList(QLatin1String("deleteCookie"), argumentList);
    }

    inline QDBusPendingReply<> deleteCookiesFromDomain(const QString &domain)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(domain);
        return asyncCallWithArgumentList(QLatin1String("deleteCookiesFromDomain"), argumentList);
    }

    inline QDBusPendingReply<> deleteSessionCookies(qlonglong windowId)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(windowId);
        return asyncCallWithArgumentList(QLatin1String("deleteSessionCookies"), argumentList);
    }

    inline QDBusPendingReply<> deleteSessionCookiesFor(const QString &fqdn, qlonglong windowId)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(fqdn) << QVariant::fromValue(windowId);
        return asyncCallWithArgumentList(QLatin1String("deleteSessionCookiesFor"), argumentList);
    }

    inline QDBusPendingReply<QStringList> findCookies(const QList<int> &fields, const QString &domain, const QString &fqdn, const QString &path, const QString &name)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(fields) << QVariant::fromValue(domain) << QVariant::fromValue(fqdn) << QVariant::fromValue(path) << QVariant::fromValue(name);
        return asyncCallWithArgumentList(QLatin1String("findCookies"), argumentList);
    }

    inline QDBusPendingReply<QString> findCookies(const QString &url, qlonglong windowId)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(url) << QVariant::fromValue(windowId);
        return asyncCallWithArgumentList(QLatin1String("findCookies"), argumentList);
    }

    inline QDBusPendingReply<QString> findDOMCookies(const QString &url, qlonglong windowId)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(url) << QVariant::fromValue(windowId);
        return asyncCallWithArgumentList(QLatin1String("findDOMCookies"), argumentList);
    }

    inline QDBusPendingReply<QString> findDOMCookies(const QString &url)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(url);
        return asyncCallWithArgumentList(QLatin1String("findDOMCookies"), argumentList);
    }

    inline QDBusPendingReply<QStringList> findDomains()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QLatin1String("findDomains"), argumentList);
    }

    inline QDBusPendingReply<QString> getDomainAdvice(const QString &url)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(url);
        return asyncCallWithArgumentList(QLatin1String("getDomainAdvice"), argumentList);
    }

    inline QDBusPendingReply<QString> listCookies(const QString &url)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(url);
        return asyncCallWithArgumentList(QLatin1String("listCookies"), argumentList);
    }

    inline QDBusPendingReply<> reloadPolicy()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QLatin1String("reloadPolicy"), argumentList);
    }

    inline QDBusPendingReply<bool> setDomainAdvice(const QString &url, const QString &advice)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(url) << QVariant::fromValue(advice);
        return asyncCallWithArgumentList(QLatin1String("setDomainAdvice"), argumentList);
    }

    inline QDBusPendingReply<> shutdown()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QLatin1String("shutdown"), argumentList);
    }

Q_SIGNALS: // SIGNALS
};

namespace org {
  namespace kde {
    typedef ::OrgKdeKCookieServerInterface KCookieServer;
  }
}
#endif
