/****************************************************************************
** Meta object code from reading C++ file 'ripple_p.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.7)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/kdelibs-4.14.38/plasma/private/effects/ripple_p.h"
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ripple_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.7. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_Plasma__RippleEffect[] = {

 // content:
       6,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       4,   49, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: signature, parameters, type, tag, flags
      29,   22,   21,   21, 0x05,
      58,   50,   21,   21, 0x05,
      87,   80,   21,   21, 0x05,

 // slots: signature, parameters, type, tag, flags
     108,   22,   21,   21, 0x0a,
     125,   50,   21,   21, 0x0a,
     143,   80,   21,   21, 0x0a,
     168,  160,   21,   21, 0x0a,

 // properties: name, type, flags
      22,  186, 0x83095103,
      50,  186, 0x83095103,
      80,  186, 0x83095103,
     160,  192, ((uint)QMetaType::QReal << 24) | 0x00095103,

       0        // eod
};

static const char qt_meta_stringdata_Plasma__RippleEffect[] = {
    "Plasma::RippleEffect\0\0offset\0"
    "offsetChanged(qint8)\0damping\0"
    "dampingChanged(qint8)\0heigth\0"
    "heigthChanged(qint8)\0setOffset(qint8)\0"
    "setDamping(qint8)\0setHeigth(qint8)\0"
    "opacity\0setOpacity(qreal)\0qint8\0qreal\0"
};

void Plasma::RippleEffect::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        RippleEffect *_t = static_cast<RippleEffect *>(_o);
        switch (_id) {
        case 0: _t->offsetChanged((*reinterpret_cast< qint8(*)>(_a[1]))); break;
        case 1: _t->dampingChanged((*reinterpret_cast< qint8(*)>(_a[1]))); break;
        case 2: _t->heigthChanged((*reinterpret_cast< qint8(*)>(_a[1]))); break;
        case 3: _t->setOffset((*reinterpret_cast< qint8(*)>(_a[1]))); break;
        case 4: _t->setDamping((*reinterpret_cast< qint8(*)>(_a[1]))); break;
        case 5: _t->setHeigth((*reinterpret_cast< qint8(*)>(_a[1]))); break;
        case 6: _t->setOpacity((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObjectExtraData Plasma::RippleEffect::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject Plasma::RippleEffect::staticMetaObject = {
    { &QGraphicsEffect::staticMetaObject, qt_meta_stringdata_Plasma__RippleEffect,
      qt_meta_data_Plasma__RippleEffect, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &Plasma::RippleEffect::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *Plasma::RippleEffect::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *Plasma::RippleEffect::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_Plasma__RippleEffect))
        return static_cast<void*>(const_cast< RippleEffect*>(this));
    return QGraphicsEffect::qt_metacast(_clname);
}

int Plasma::RippleEffect::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsEffect::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
#ifndef QT_NO_PROPERTIES
      else if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< qint8*>(_v) = offset(); break;
        case 1: *reinterpret_cast< qint8*>(_v) = damping(); break;
        case 2: *reinterpret_cast< qint8*>(_v) = heigth(); break;
        case 3: *reinterpret_cast< qreal*>(_v) = opacity(); break;
        }
        _id -= 4;
    } else if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: setOffset(*reinterpret_cast< qint8*>(_v)); break;
        case 1: setDamping(*reinterpret_cast< qint8*>(_v)); break;
        case 2: setHeigth(*reinterpret_cast< qint8*>(_v)); break;
        case 3: setOpacity(*reinterpret_cast< qreal*>(_v)); break;
        }
        _id -= 4;
    } else if (_c == QMetaObject::ResetProperty) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void Plasma::RippleEffect::offsetChanged(qint8 _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Plasma::RippleEffect::dampingChanged(qint8 _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Plasma::RippleEffect::heigthChanged(qint8 _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_END_MOC_NAMESPACE
