#include <kdialog.h>
#include <klocale.h>

/********************************************************************************
** Form generated from reading UI file 'UploadDialog.ui'
**
** Created by: Qt User Interface Compiler version 4.8.7
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_UPLOADDIALOG_H
#define UI_UPLOADDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QFormLayout>
#include <QtGui/QHeaderView>
#include <QtGui/QLabel>
#include <QtGui/QWidget>
#include "kcombobox.h"
#include "klineedit.h"
#include "ktextedit.h"
#include "ktitlewidget.h"
#include "kurlrequester.h"

QT_BEGIN_NAMESPACE

class Ui_UploadDialog
{
public:
    QFormLayout *formLayout;
    QLabel *label;
    KLineEdit *mAuthorEdit;
    QLabel *label_2;
    KLineEdit *mEmailEdit;
    QLabel *label_4;
    KLineEdit *mNameEdit;
    QLabel *label_5;
    KLineEdit *mVersionEdit;
    QLabel *label_6;
    KComboBox *mLicenseCombo;
    QLabel *label_7;
    KUrlRequester *mPreviewUrl;
    QLabel *label_10;
    KTextEdit *mSummaryEdit;
    KComboBox *mLanguageCombo;
    QLabel *label_9;
    QLabel *label_3;
    QLabel *label_8;
    QLabel *label_11;
    KTitleWidget *mTitleWidget;

    void setupUi(QWidget *UploadDialog)
    {
        if (UploadDialog->objectName().isEmpty())
            UploadDialog->setObjectName(QString::fromUtf8("UploadDialog"));
        UploadDialog->resize(513, 548);
        formLayout = new QFormLayout(UploadDialog);
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        label = new QLabel(UploadDialog);
        label->setObjectName(QString::fromUtf8("label"));

        formLayout->setWidget(2, QFormLayout::LabelRole, label);

        mAuthorEdit = new KLineEdit(UploadDialog);
        mAuthorEdit->setObjectName(QString::fromUtf8("mAuthorEdit"));

        formLayout->setWidget(2, QFormLayout::FieldRole, mAuthorEdit);

        label_2 = new QLabel(UploadDialog);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        formLayout->setWidget(3, QFormLayout::LabelRole, label_2);

        mEmailEdit = new KLineEdit(UploadDialog);
        mEmailEdit->setObjectName(QString::fromUtf8("mEmailEdit"));

        formLayout->setWidget(3, QFormLayout::FieldRole, mEmailEdit);

        label_4 = new QLabel(UploadDialog);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        formLayout->setWidget(5, QFormLayout::LabelRole, label_4);

        mNameEdit = new KLineEdit(UploadDialog);
        mNameEdit->setObjectName(QString::fromUtf8("mNameEdit"));

        formLayout->setWidget(5, QFormLayout::FieldRole, mNameEdit);

        label_5 = new QLabel(UploadDialog);
        label_5->setObjectName(QString::fromUtf8("label_5"));

        formLayout->setWidget(6, QFormLayout::LabelRole, label_5);

        mVersionEdit = new KLineEdit(UploadDialog);
        mVersionEdit->setObjectName(QString::fromUtf8("mVersionEdit"));

        formLayout->setWidget(6, QFormLayout::FieldRole, mVersionEdit);

        label_6 = new QLabel(UploadDialog);
        label_6->setObjectName(QString::fromUtf8("label_6"));

        formLayout->setWidget(7, QFormLayout::LabelRole, label_6);

        mLicenseCombo = new KComboBox(UploadDialog);
        mLicenseCombo->setObjectName(QString::fromUtf8("mLicenseCombo"));
        mLicenseCombo->setEditable(true);

        formLayout->setWidget(7, QFormLayout::FieldRole, mLicenseCombo);

        label_7 = new QLabel(UploadDialog);
        label_7->setObjectName(QString::fromUtf8("label_7"));

        formLayout->setWidget(8, QFormLayout::LabelRole, label_7);

        mPreviewUrl = new KUrlRequester(UploadDialog);
        mPreviewUrl->setObjectName(QString::fromUtf8("mPreviewUrl"));

        formLayout->setWidget(8, QFormLayout::FieldRole, mPreviewUrl);

        label_10 = new QLabel(UploadDialog);
        label_10->setObjectName(QString::fromUtf8("label_10"));

        formLayout->setWidget(12, QFormLayout::LabelRole, label_10);

        mSummaryEdit = new KTextEdit(UploadDialog);
        mSummaryEdit->setObjectName(QString::fromUtf8("mSummaryEdit"));

        formLayout->setWidget(10, QFormLayout::FieldRole, mSummaryEdit);

        mLanguageCombo = new KComboBox(UploadDialog);
        mLanguageCombo->setObjectName(QString::fromUtf8("mLanguageCombo"));

        formLayout->setWidget(12, QFormLayout::FieldRole, mLanguageCombo);

        label_9 = new QLabel(UploadDialog);
        label_9->setObjectName(QString::fromUtf8("label_9"));

        formLayout->setWidget(11, QFormLayout::SpanningRole, label_9);

        label_3 = new QLabel(UploadDialog);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        formLayout->setWidget(4, QFormLayout::SpanningRole, label_3);

        label_8 = new QLabel(UploadDialog);
        label_8->setObjectName(QString::fromUtf8("label_8"));

        formLayout->setWidget(10, QFormLayout::LabelRole, label_8);

        label_11 = new QLabel(UploadDialog);
        label_11->setObjectName(QString::fromUtf8("label_11"));

        formLayout->setWidget(1, QFormLayout::SpanningRole, label_11);

        mTitleWidget = new KTitleWidget(UploadDialog);
        mTitleWidget->setObjectName(QString::fromUtf8("mTitleWidget"));

        formLayout->setWidget(0, QFormLayout::SpanningRole, mTitleWidget);

#ifndef UI_QT_NO_SHORTCUT
        label->setBuddy(mAuthorEdit);
        label_2->setBuddy(mEmailEdit);
        label_4->setBuddy(mNameEdit);
        label_5->setBuddy(mVersionEdit);
        label_6->setBuddy(mLicenseCombo);
        label_7->setBuddy(mPreviewUrl);
        label_10->setBuddy(mLanguageCombo);
        label_8->setBuddy(mSummaryEdit);
#endif // QT_NO_SHORTCUT
        QWidget::setTabOrder(mAuthorEdit, mEmailEdit);
        QWidget::setTabOrder(mEmailEdit, mNameEdit);
        QWidget::setTabOrder(mNameEdit, mVersionEdit);
        QWidget::setTabOrder(mVersionEdit, mLicenseCombo);
        QWidget::setTabOrder(mLicenseCombo, mPreviewUrl);

        retranslateUi(UploadDialog);

        QMetaObject::connectSlotsByName(UploadDialog);
    } // setupUi

    void retranslateUi(QWidget *UploadDialog)
    {
        UploadDialog->setWindowTitle(tr2i18n("Share Hot New Stuff", 0));
        label->setText(tr2i18n("Author:", 0));
        label_2->setText(tr2i18n("Email address:", 0));
        label_4->setText(tr2i18n("Name:", 0));
        label_5->setText(tr2i18n("Version:", 0));
        label_6->setText(tr2i18n("License:", 0));
        mLicenseCombo->clear();
        mLicenseCombo->insertItems(0, QStringList()
         << tr2i18n("GPL", 0)
         << tr2i18n("LGPL", 0)
         << tr2i18n("BSD", 0)
        );
        label_7->setText(tr2i18n("Preview URL:", 0));
        label_10->setText(tr2i18n("Language:", 0));
        label_9->setText(tr2i18n("In which language did you describe the above?", 0));
        label_3->setText(tr2i18n("Please describe your upload.", 0));
        label_8->setText(tr2i18n("Summary:", 0));
        label_11->setText(tr2i18n("Please give some information about yourself.", 0));
    } // retranslateUi

};

namespace Ui {
    class UploadDialog: public Ui_UploadDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UPLOADDIALOG_H

