// This file is generated by kconfig_compiler from libplasma-theme-global.kcfg.
// All changes you do to this file will be lost.

#include "libplasma-theme-global.h"

ThemeConfig::ThemeConfig(  )
  : KConfigSkeleton( QLatin1String( "plasmarc" ) )
{
  setCurrentGroup( QLatin1String( "CachePolicies" ) );

  KConfigSkeleton::ItemBool  *itemCacheTheme;
  itemCacheTheme = new KConfigSkeleton::ItemBool( currentGroup(), QLatin1String( "CacheTheme" ), mCacheTheme, true );
  addItem( itemCacheTheme, QLatin1String( "CacheTheme" ) );
  KConfigSkeleton::ItemInt  *itemThemeCacheKb;
  itemThemeCacheKb = new KConfigSkeleton::ItemInt( currentGroup(), QLatin1String( "ThemeCacheKb" ), mThemeCacheKb, 81920 );
  addItem( itemThemeCacheKb, QLatin1String( "ThemeCacheKb" ) );
}

ThemeConfig::~ThemeConfig()
{
}

