/**
* This file was autogenerated by makekdewidgets. Any changes will be lost!
* The generated code in this file is licensed under the same license that the
* input file.
*/
#include <QIcon>
#include <QtDesigner/QDesignerContainerExtension>
#include <QDesignerCustomWidgetInterface>
#include <qplugin.h>
#include <qdebug.h>

#include <kcomponentdata.h>
#include <classpreviews.h>
#include <ktimezone.h>
#include <kdialog.h>
#include <keditlistwidget.h>
#include <kpixmapregionselectorwidget.h>
#include <klanguagebutton.h>
#include <kicondialog.h>
#include <ktextbrowser.h>
#include <krichtextedit.h>
#include <kbuttongroup.h>
#include <kselector.h>
#include <kdatetable.h>
#include <kcolorcombo.h>
#include <knuminput.h>
#include <kkeysequencewidget.h>
#include <kcapacitybar.h>
#include <ktimecombobox.h>
#include <kfilterproxysearchline.h>
#include <kfontcombobox.h>
#include <kdatecombobox.h>
#include <kcolorbutton.h>
#include <kpageview.h>
#include <kfilemetadatawidget.h>
#include <ktabbar.h>
#include <kfontrequester.h>
#include <kdialogbuttonbox.h>
#include <kdatetimeedit.h>
#include <kfontdialog.h>
#include <kdatepicker.h>
#include <kcombobox.h>
#include <klistwidgetsearchline.h>
#include <kurlrequester.h>
#include <kdatewidget.h>
#include <kcategorizedview.h>
#include <klineedit.h>
#include <kseparator.h>
#include <kcharselect.h>
#include <kurllabel.h>
#include <kcolorvalueselector.h>
#include <kpushbutton.h>
#include <kled.h>
#include <kplotwidget.h>
#include <krestrictedline.h>
#include <knuminput.h>
#include <ktreewidgetsearchline.h>
#include <ktitlewidget.h>
#include <kpagewidget.h>
#include <ktextedit.h>
#include <klistwidget.h>
#include <krichtextwidget.h>
#include <kpixmapsequencewidget.h>
#include <ktimezonewidget.h>
#include <kmultitabbar.h>
#include <kurlrequester.h>
#include <knuminput.h>
#include <kshortcutwidget.h>
#include <ktabwidget.h>
#include <kdatetimewidget.h>
#include <kruler.h>
#include <ksqueezedtextlabel.h>
#include <kimagefilepreview.h>
#include <kratingwidget.h>
#include <kanimatedbutton.h>
#include <kcolordialog.h>
#include <ktreewidgetsearchline.h>
#include <kshortcutseditor.h>
#include <kcolordialog.h>
#include <sonnet/dictionarycombobox.h>
#include <kactionselector.h>
#include <khistorycombobox.h>


class KDialogPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KDialogPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KDialogPlugin() {}
	
	bool isContainer() const { return true; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kdialog.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Container (KDE)"); }
	QString includeFile() const { return QLatin1String("kdialog.h"); }
	QString name() const { return QLatin1String("KDialog"); }
	QString toolTip() const { return QLatin1String("Class for simple Dialogs (KDE)"); }
	QString whatsThis() const { return QLatin1String("KDialog Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KDialogPreview( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KEditListWidgetPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KEditListWidgetPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KEditListWidgetPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/keditlistwidget.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Views (KDE)"); }
	QString includeFile() const { return QLatin1String("keditlistwidget.h"); }
	QString name() const { return QLatin1String("KEditListWidget"); }
	QString toolTip() const { return QLatin1String("Fullfeatured edit box with buttons (KDE)"); }
	QString whatsThis() const { return QLatin1String("KEditListWidget Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KEditListWidget( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KPixmapRegionSelectorWidgetPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KPixmapRegionSelectorWidgetPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KPixmapRegionSelectorWidgetPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kpixmapregionselectorwidget.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Graphics (KDE)"); }
	QString includeFile() const { return QLatin1String("kpixmapregionselectorwidget.h"); }
	QString name() const { return QLatin1String("KPixmapRegionSelectorWidget"); }
	QString toolTip() const { return QLatin1String("Pixmap Region Selector (KDE)"); }
	QString whatsThis() const { return QLatin1String("KPixmapRegionSelectorWidget Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KPixmapRegionSelectorWidget( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KLanguageButtonPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KLanguageButtonPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KLanguageButtonPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/klanguagebutton.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Buttons (KDE)"); }
	QString includeFile() const { return QLatin1String("klanguagebutton.h"); }
	QString name() const { return QLatin1String("KLanguageButton"); }
	QString toolTip() const { return QLatin1String("Language Button (KDE)"); }
	QString whatsThis() const { return QLatin1String("KDE's language button."); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KLanguageButton( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KIconButtonPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KIconButtonPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KIconButtonPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kiconbutton.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Buttons (KDE)"); }
	QString includeFile() const { return QLatin1String("kicondialog.h"); }
	QString name() const { return QLatin1String("KIconButton"); }
	QString toolTip() const { return QLatin1String("Button for selecting an icon (KDE)"); }
	QString whatsThis() const { return QLatin1String("KIconButton Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KIconButton( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KTextBrowserPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KTextBrowserPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KTextBrowserPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/ktextbrowser.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Display (KDE)"); }
	QString includeFile() const { return QLatin1String("ktextbrowser.h"); }
	QString name() const { return QLatin1String("KTextBrowser"); }
	QString toolTip() const { return QLatin1String("Improved QTextBrowser (KDE)"); }
	QString whatsThis() const { return QLatin1String("An improved version of the QTextBrowser with mail or system browser invocation support"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KTextBrowser( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KRichTextEditPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KRichTextEditPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KRichTextEditPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/krichtextedit.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Input (KDE)"); }
	QString includeFile() const { return QLatin1String("krichtextedit.h"); }
	QString name() const { return QLatin1String("KRichTextEdit"); }
	QString toolTip() const { return QLatin1String("Rich Text Editor (KDE)"); }
	QString whatsThis() const { return QLatin1String("KRichTextEdit Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KRichTextEdit( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KButtonGroupPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KButtonGroupPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KButtonGroupPlugin() {}
	
	bool isContainer() const { return true; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kbuttongroup.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Container (KDE)"); }
	QString includeFile() const { return QLatin1String("kbuttongroup.h"); }
	QString name() const { return QLatin1String("KButtonGroup"); }
	QString toolTip() const { return QLatin1String("Group of radio buttons with index selection."); }
	QString whatsThis() const { return QLatin1String("KButtonGroup Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KButtonGroup( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KGradientSelectorPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KGradientSelectorPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KGradientSelectorPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kgradientselector.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Input (KDE)"); }
	QString includeFile() const { return QLatin1String("kselector.h"); }
	QString name() const { return QLatin1String("KGradientSelector"); }
	QString toolTip() const { return QLatin1String("Gradient Color Selector (KDE)"); }
	QString whatsThis() const { return QLatin1String("KGradientSelector Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KGradientSelector( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KDateTablePlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KDateTablePlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KDateTablePlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kdatetable.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Date/Time (KDE)"); }
	QString includeFile() const { return QLatin1String("kdatetable.h"); }
	QString name() const { return QLatin1String("KDateTable"); }
	QString toolTip() const { return QLatin1String("KDateTable Widget"); }
	QString whatsThis() const { return QLatin1String("KDateTable Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KDateTable( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KColorComboPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KColorComboPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KColorComboPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kcolorcombo.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Graphics (KDE)"); }
	QString includeFile() const { return QLatin1String("kcolorcombo.h"); }
	QString name() const { return QLatin1String("KColorCombo"); }
	QString toolTip() const { return QLatin1String("Color Combo Box (KDE)"); }
	QString whatsThis() const { return QLatin1String("KColorCombo Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KColorCombo( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KDoubleNumInputPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KDoubleNumInputPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KDoubleNumInputPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kdoublenuminput.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Input (KDE)"); }
	QString includeFile() const { return QLatin1String("knuminput.h"); }
	QString name() const { return QLatin1String("KDoubleNumInput"); }
	QString toolTip() const { return QLatin1String("Floating Point Number Input Widget (KDE)"); }
	QString whatsThis() const { return QLatin1String("An input widget for floating point numbers, consisting of a spinbox and a slider."); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KDoubleNumInput( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KKeySequenceWidgetPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KKeySequenceWidgetPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KKeySequenceWidgetPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kkeysequencewidget.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Buttons (KDE)"); }
	QString includeFile() const { return QLatin1String("kkeysequencewidget.h"); }
	QString name() const { return QLatin1String("KKeySequenceWidget"); }
	QString toolTip() const { return QLatin1String("QKeySequence picker widget. (KDE)"); }
	QString whatsThis() const { return QLatin1String("A widget to pick a QKeySequence."); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KKeySequenceWidget( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KCapacityBarPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KCapacityBarPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KCapacityBarPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kcapacitybar.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Display (KDE)"); }
	QString includeFile() const { return QLatin1String("kcapacitybar.h"); }
	QString name() const { return QLatin1String("KCapacityBar"); }
	QString toolTip() const { return QLatin1String("Capacity Bar (KDE)"); }
	QString whatsThis() const { return QLatin1String("KCapacityBar Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KCapacityBar(KCapacityBar::DrawTextOutline, parent);
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KTimeComboBoxPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KTimeComboBoxPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KTimeComboBoxPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/ktimecombobox.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Date/Time (KDE)"); }
	QString includeFile() const { return QLatin1String("ktimecombobox.h"); }
	QString name() const { return QLatin1String("KTimeComboBox"); }
	QString toolTip() const { return QLatin1String("This widget can be used to display or edit the time. (KDE)"); }
	QString whatsThis() const { return QLatin1String("KTimeComboBox Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KTimeComboBox( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KFilterProxySearchLinePlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KFilterProxySearchLinePlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KFilterProxySearchLinePlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kfilterproxysearchline.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Input (KDE)"); }
	QString includeFile() const { return QLatin1String("kfilterproxysearchline.h"); }
	QString name() const { return QLatin1String("KFilterProxySearchLine"); }
	QString toolTip() const { return QLatin1String("Filter proxy search line"); }
	QString whatsThis() const { return QLatin1String("A search line widget which works on a QSortFilterProxyModel"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KFilterProxySearchLine( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KFontComboBoxPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KFontComboBoxPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KFontComboBoxPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kfontcombobox.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Input (KDE)"); }
	QString includeFile() const { return QLatin1String("kfontcombobox.h"); }
	QString name() const { return QLatin1String("KFontComboBox"); }
	QString toolTip() const { return QLatin1String("Font Combo Box (KDE)"); }
	QString whatsThis() const { return QLatin1String("A combo box for font selection with language-specific previews"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KFontComboBox( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KDateComboBoxPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KDateComboBoxPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KDateComboBoxPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kdatecombobox.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Date/Time (KDE)"); }
	QString includeFile() const { return QLatin1String("kdatecombobox.h"); }
	QString name() const { return QLatin1String("KDateComboBox"); }
	QString toolTip() const { return QLatin1String("This widget can be used to display or edit the date. (KDE)"); }
	QString whatsThis() const { return QLatin1String("KDateComboBox Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KDateComboBox( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KColorButtonPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KColorButtonPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KColorButtonPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kcolorbutton.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Graphics (KDE)"); }
	QString includeFile() const { return QLatin1String("kcolorbutton.h"); }
	QString name() const { return QLatin1String("KColorButton"); }
	QString toolTip() const { return QLatin1String("Color Chooser Button (KDE)"); }
	QString whatsThis() const { return QLatin1String("A button that allows selecting a color"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KColorButton( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KPageViewPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KPageViewPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KPageViewPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kpageview.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Views (KDE)"); }
	QString includeFile() const { return QLatin1String("kpageview.h"); }
	QString name() const { return QLatin1String("KPageView"); }
	QString toolTip() const { return QLatin1String("Paged View (KDE)"); }
	QString whatsThis() const { return QLatin1String("A base class which can handle multiple pages."); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KPageView( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KFileMetaDataWidgetPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KFileMetaDataWidgetPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KFileMetaDataWidgetPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kfilemetadatawidget.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Display (KDE)"); }
	QString includeFile() const { return QLatin1String("kfilemetadatawidget.h"); }
	QString name() const { return QLatin1String("KFileMetaDataWidget"); }
	QString toolTip() const { return QLatin1String("File Meta Data Display"); }
	QString whatsThis() const { return QLatin1String("Shows the meta data of one or more file items."); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KFileMetaDataWidget( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KTabBarPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KTabBarPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KTabBarPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/ktabbar.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Display (KDE)"); }
	QString includeFile() const { return QLatin1String("ktabbar.h"); }
	QString name() const { return QLatin1String("KTabBar"); }
	QString toolTip() const { return QLatin1String("KDE Tab Bar"); }
	QString whatsThis() const { return QLatin1String("A QTabBar with extended features."); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KTabBar( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KFontRequesterPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KFontRequesterPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KFontRequesterPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kfontrequester.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Input (KDE)"); }
	QString includeFile() const { return QLatin1String("kfontrequester.h"); }
	QString name() const { return QLatin1String("KFontRequester"); }
	QString toolTip() const { return QLatin1String("Font Requester (KDE)"); }
	QString whatsThis() const { return QLatin1String("A compact font selection and preview widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KFontRequester( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KDialogButtonBoxPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KDialogButtonBoxPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KDialogButtonBoxPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kdialogbuttonbox.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QLatin1String("<ui><widget class=\"KDialogButtonBox\"><property name=\"standardButtons\"><set>QDialogButtonBox::Ok|QDialogButtonBox::Cancel</set></property></widget></ui>"); }
	QString group() const { return QLatin1String("Buttons (KDE)"); }
	QString includeFile() const { return QLatin1String("kdialogbuttonbox.h"); }
	QString name() const { return QLatin1String("KDialogButtonBox"); }
	QString toolTip() const { return QLatin1String("Button Box (KDE)"); }
	QString whatsThis() const { return QLatin1String("KDE's button box."); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KDialogButtonBox( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KDateTimeEditPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KDateTimeEditPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KDateTimeEditPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kdatetimeedit.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Date/Time (KDE)"); }
	QString includeFile() const { return QLatin1String("kdatetimeedit.h"); }
	QString name() const { return QLatin1String("KDateTimeEdit"); }
	QString toolTip() const { return QLatin1String("This widget can be used to display or edit the date and time. (KDE)"); }
	QString whatsThis() const { return QLatin1String("KDateTimeEdit Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KDateTimeEdit( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KFontChooserPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KFontChooserPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KFontChooserPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kfontchooser.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Input (KDE)"); }
	QString includeFile() const { return QLatin1String("kfontdialog.h"); }
	QString name() const { return QLatin1String("KFontChooser"); }
	QString toolTip() const { return QLatin1String("Font Chooser (KDE)"); }
	QString whatsThis() const { return QLatin1String("A font type, size and style selection widget complete with preview"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KFontChooser( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KDatePickerPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KDatePickerPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KDatePickerPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kdatepicker.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Date/Time (KDE)"); }
	QString includeFile() const { return QLatin1String("kdatepicker.h"); }
	QString name() const { return QLatin1String("KDatePicker"); }
	QString toolTip() const { return QLatin1String("A date selection widget (KDE)"); }
	QString whatsThis() const { return QLatin1String("Provides a widget for calendar date input"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KDatePicker( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KComboBoxPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KComboBoxPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KComboBoxPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kcombobox.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Input (KDE)"); }
	QString includeFile() const { return QLatin1String("kcombobox.h"); }
	QString name() const { return QLatin1String("KComboBox"); }
	QString toolTip() const { return QLatin1String("Combo Box (KDE)"); }
	QString whatsThis() const { return QLatin1String("KComboBox Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KComboBox( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KListWidgetSearchLinePlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KListWidgetSearchLinePlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KListWidgetSearchLinePlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/klistwidgetsearchline.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Input (KDE)"); }
	QString includeFile() const { return QLatin1String("klistwidgetsearchline.h"); }
	QString name() const { return QLatin1String("KListWidgetSearchLine"); }
	QString toolTip() const { return QLatin1String("QListWidget Search Line (KDE)"); }
	QString whatsThis() const { return QLatin1String("KListWidgetSearchLine Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KListWidgetSearchLine(parent, 0);
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KUrlComboRequesterPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KUrlComboRequesterPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KUrlComboRequesterPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kurlcomborequester.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Input (KDE)"); }
	QString includeFile() const { return QLatin1String("kurlrequester.h"); }
	QString name() const { return QLatin1String("KUrlComboRequester"); }
	QString toolTip() const { return QLatin1String("URL Requester (KDE)"); }
	QString whatsThis() const { return QLatin1String("A widget holding a combobox and a pushbutton. A \"file open\" dialog opened when clicking on the pushbutton"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KUrlComboRequester( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KDateWidgetPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KDateWidgetPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KDateWidgetPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kdatewidget.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Date/Time (KDE)"); }
	QString includeFile() const { return QLatin1String("kdatewidget.h"); }
	QString name() const { return QLatin1String("KDateWidget"); }
	QString toolTip() const { return QLatin1String("Date preview (KDE)"); }
	QString whatsThis() const { return QLatin1String("KDateWidget Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KDateWidget( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KCategorizedViewPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KCategorizedViewPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KCategorizedViewPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kcategorizedview.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Views (KDE)"); }
	QString includeFile() const { return QLatin1String("kcategorizedview.h"); }
	QString name() const { return QLatin1String("KCategorizedView"); }
	QString toolTip() const { return QLatin1String("Categorized Item View (KDE)"); }
	QString whatsThis() const { return QLatin1String("Item view for listing items in a categorized fashion optionally."); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KCategorizedView( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KLineEditPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KLineEditPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KLineEditPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/klineedit.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Input (KDE)"); }
	QString includeFile() const { return QLatin1String("klineedit.h"); }
	QString name() const { return QLatin1String("KLineEdit"); }
	QString toolTip() const { return QLatin1String("Line Edit (KDE)"); }
	QString whatsThis() const { return QLatin1String("KLineEdit Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KLineEdit( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KSeparatorPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KSeparatorPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KSeparatorPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kseparator.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Display (KDE)"); }
	QString includeFile() const { return QLatin1String("kseparator.h"); }
	QString name() const { return QLatin1String("KSeparator"); }
	QString toolTip() const { return QLatin1String("Standard horizontal or vertical separator."); }
	QString whatsThis() const { return QLatin1String("KSeparator Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KSeparator( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KCharSelectPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KCharSelectPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KCharSelectPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kcharselect.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Graphics (KDE)"); }
	QString includeFile() const { return QLatin1String("kcharselect.h"); }
	QString name() const { return QLatin1String("KCharSelect"); }
	QString toolTip() const { return QLatin1String("Character Selection Widget (KDE)"); }
	QString whatsThis() const { return QLatin1String("A widget that allows selection a character out ot a table"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KCharSelect(parent, 0);
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KUrlLabelPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KUrlLabelPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KUrlLabelPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kurllabel.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Display (KDE)"); }
	QString includeFile() const { return QLatin1String("kurllabel.h"); }
	QString name() const { return QLatin1String("KUrlLabel"); }
	QString toolTip() const { return QLatin1String("URL Label (KDE)"); }
	QString whatsThis() const { return QLatin1String("KUrlLabel Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KUrlLabel("KUrlLabel", QString(), parent);
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KColorValueSelectorPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KColorValueSelectorPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KColorValueSelectorPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kcolorvalueselector.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Input (KDE)"); }
	QString includeFile() const { return QLatin1String("kcolorvalueselector.h"); }
	QString name() const { return QLatin1String("KColorValueSelector"); }
	QString toolTip() const { return QLatin1String("Color Value Selector (KDE)"); }
	QString whatsThis() const { return QLatin1String("KColorValueSelector Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KColorValueSelector( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KPushButtonPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KPushButtonPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KPushButtonPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kpushbutton.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Buttons (KDE)"); }
	QString includeFile() const { return QLatin1String("kpushbutton.h"); }
	QString name() const { return QLatin1String("KPushButton"); }
	QString toolTip() const { return QLatin1String("Improved QPushButton (KDE)"); }
	QString whatsThis() const { return QLatin1String("An improved QPushButton to follow KDE settings"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KPushButton( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KLedPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KLedPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KLedPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kled.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Display (KDE)"); }
	QString includeFile() const { return QLatin1String("kled.h"); }
	QString name() const { return QLatin1String("KLed"); }
	QString toolTip() const { return QLatin1String("LED Widget (KDE)"); }
	QString whatsThis() const { return QLatin1String("A widget showing a light emitter diode"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KLed( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KPlotWidgetPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KPlotWidgetPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KPlotWidgetPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kplotwidget.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Plot (KDE)"); }
	QString includeFile() const { return QLatin1String("kplotwidget.h"); }
	QString name() const { return QLatin1String("KPlotWidget"); }
	QString toolTip() const { return QLatin1String("A plot widget"); }
	QString whatsThis() const { return QLatin1String("A widget for easily making 2-dimensional plots"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KPlotWidget( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KRestrictedLinePlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KRestrictedLinePlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KRestrictedLinePlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/krestrictedline.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Input (KDE)"); }
	QString includeFile() const { return QLatin1String("krestrictedline.h"); }
	QString name() const { return QLatin1String("KRestrictedLine"); }
	QString toolTip() const { return QLatin1String("Line Edit for restricted input (KDE)"); }
	QString whatsThis() const { return QLatin1String("KRestrictedLine Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KRestrictedLine( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KIntSpinBoxPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KIntSpinBoxPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KIntSpinBoxPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kintspinbox.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Input (KDE)"); }
	QString includeFile() const { return QLatin1String("knuminput.h"); }
	QString name() const { return QLatin1String("KIntSpinBox"); }
	QString toolTip() const { return QLatin1String("Enhanced Spinbox for Integer Values (KDE)"); }
	QString whatsThis() const { return QLatin1String("KIntSpinBox Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KIntSpinBox( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KTreeWidgetSearchLinePlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KTreeWidgetSearchLinePlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KTreeWidgetSearchLinePlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/ktreewidgetsearchline.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Input (KDE)"); }
	QString includeFile() const { return QLatin1String("ktreewidgetsearchline.h"); }
	QString name() const { return QLatin1String("KTreeWidgetSearchLine"); }
	QString toolTip() const { return QLatin1String("QTreeWidget Search Line (KDE)"); }
	QString whatsThis() const { return QLatin1String("KTreeWidgetSearchLine Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KTreeWidgetSearchLine(parent, 0);
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KTitleWidgetPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KTitleWidgetPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KTitleWidgetPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/ktitlewidget.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Display (KDE)"); }
	QString includeFile() const { return QLatin1String("ktitlewidget.h"); }
	QString name() const { return QLatin1String("KTitleWidget"); }
	QString toolTip() const { return QLatin1String("Title box with label and icon"); }
	QString whatsThis() const { return QLatin1String("A styled frame to be used in title positions in dialogs and other widgets"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KTitleWidget( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KPageWidgetPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KPageWidgetPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KPageWidgetPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kpagewidget.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Views (KDE)"); }
	QString includeFile() const { return QLatin1String("kpagewidget.h"); }
	QString name() const { return QLatin1String("KPageWidget"); }
	QString toolTip() const { return QLatin1String("Paged Widget (KDE)"); }
	QString whatsThis() const { return QLatin1String("Page widget with many layouts (faces)."); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KPageWidget( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KTextEditPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KTextEditPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KTextEditPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/ktextedit.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Input (KDE)"); }
	QString includeFile() const { return QLatin1String("ktextedit.h"); }
	QString name() const { return QLatin1String("KTextEdit"); }
	QString toolTip() const { return QLatin1String("Improved QTextEdit (KDE)"); }
	QString whatsThis() const { return QLatin1String("An improved version of the QTextEdit with mail or system browser invocation support"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KTextEdit( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KListWidgetPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KListWidgetPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KListWidgetPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/klistwidget.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Views (KDE)"); }
	QString includeFile() const { return QLatin1String("klistwidget.h"); }
	QString name() const { return QLatin1String("KListWidget"); }
	QString toolTip() const { return QLatin1String("Extended List Widget (KDE)"); }
	QString whatsThis() const { return QLatin1String("An improved version of the QListWidget that follows KDE settings"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KListWidget( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KRichTextWidgetPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KRichTextWidgetPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KRichTextWidgetPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/krichtextwidget.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Input (KDE)"); }
	QString includeFile() const { return QLatin1String("krichtextwidget.h"); }
	QString name() const { return QLatin1String("KRichTextWidget"); }
	QString toolTip() const { return QLatin1String("Rich Text Widget (KDE)"); }
	QString whatsThis() const { return QLatin1String("KRichTextWidget Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KRichTextWidget( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KPixmapSequenceWidgetPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KPixmapSequenceWidgetPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KPixmapSequenceWidgetPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kpixmapsequencewidget.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Display (KDE)"); }
	QString includeFile() const { return QLatin1String("kpixmapsequencewidget.h"); }
	QString name() const { return QLatin1String("KPixmapSequenceWidget"); }
	QString toolTip() const { return QLatin1String("Pixmap Sequence Widget (KDE)"); }
	QString whatsThis() const { return QLatin1String("A simple widget showing a fixed size pixmap sequence."); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KPixmapSequenceWidget( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KTimeZoneWidgetPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KTimeZoneWidgetPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KTimeZoneWidgetPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/ktimezonewidget.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Input (KDE)"); }
	QString includeFile() const { return QLatin1String("ktimezonewidget.h"); }
	QString name() const { return QLatin1String("KTimeZoneWidget"); }
	QString toolTip() const { return QLatin1String("This widget can be used to display or allow user selection of timezones. (KDE)"); }
	QString whatsThis() const { return QLatin1String("KTimeZoneWidget Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KTimeZoneWidget(parent, new KTimeZones());
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KMultiTabBarPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KMultiTabBarPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KMultiTabBarPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kmultitabbar.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Display (KDE)"); }
	QString includeFile() const { return QLatin1String("kmultitabbar.h"); }
	QString name() const { return QLatin1String("KMultiTabBar"); }
	QString toolTip() const { return QLatin1String("Multi Tab Bar Widget (KDE)"); }
	QString whatsThis() const { return QLatin1String("A Widget for horizontal and vertical tabs."); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KMultiTabBar(KMultiTabBar::Left, parent);
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KUrlRequesterPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KUrlRequesterPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KUrlRequesterPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kurlrequester.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Input (KDE)"); }
	QString includeFile() const { return QLatin1String("kurlrequester.h"); }
	QString name() const { return QLatin1String("KUrlRequester"); }
	QString toolTip() const { return QLatin1String("URL Requester (KDE)"); }
	QString whatsThis() const { return QLatin1String("A widget holding a lineedit and a pushbutton. A \"file open\" dialog opened when clicking on the pushbutton"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KUrlRequester( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KIntNumInputPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KIntNumInputPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KIntNumInputPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kintnuminput.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Input (KDE)"); }
	QString includeFile() const { return QLatin1String("knuminput.h"); }
	QString name() const { return QLatin1String("KIntNumInput"); }
	QString toolTip() const { return QLatin1String("Integer Input Widget (KDE)"); }
	QString whatsThis() const { return QLatin1String("An input widget for integer numbers, consisting of a spinbox and a slider."); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KIntNumInput( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KShortcutWidgetPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KShortcutWidgetPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KShortcutWidgetPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kshortcutwidget.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Buttons (KDE)"); }
	QString includeFile() const { return QLatin1String("kshortcutwidget.h"); }
	QString name() const { return QLatin1String("KShortcutWidget"); }
	QString toolTip() const { return QLatin1String("Shortcut picker widget. (KDE)"); }
	QString whatsThis() const { return QLatin1String("A widget to pick a KShortcut."); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KShortcutWidget( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KTabWidgetPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KTabWidgetPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KTabWidgetPlugin() {}
	
	bool isContainer() const { return true; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/ktabwidget.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Container (KDE)"); }
	QString includeFile() const { return QLatin1String("ktabwidget.h"); }
	QString name() const { return QLatin1String("KTabWidget"); }
	QString toolTip() const { return QLatin1String("KDE tab widget container."); }
	QString whatsThis() const { return QLatin1String("KTabWidget Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KTabWidget( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KDateTimeWidgetPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KDateTimeWidgetPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KDateTimeWidgetPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kdatetimewidget.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Date/Time (KDE)"); }
	QString includeFile() const { return QLatin1String("kdatetimewidget.h"); }
	QString name() const { return QLatin1String("KDateTimeWidget"); }
	QString toolTip() const { return QLatin1String("This widget can be used to display or allow user selection of date and time. (KDE)"); }
	QString whatsThis() const { return QLatin1String("KDateTimeWidget Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KDateTimeWidget( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KRulerPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KRulerPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KRulerPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kruler.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Display (KDE)"); }
	QString includeFile() const { return QLatin1String("kruler.h"); }
	QString name() const { return QLatin1String("KRuler"); }
	QString toolTip() const { return QLatin1String("Measuring Ruler Widget (KDE)"); }
	QString whatsThis() const { return QLatin1String("A measuring ruler widget as seen in KWord for page widths and heights"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KRuler( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KSqueezedTextLabelPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KSqueezedTextLabelPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KSqueezedTextLabelPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/ksqueezedtextlabel.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Display (KDE)"); }
	QString includeFile() const { return QLatin1String("ksqueezedtextlabel.h"); }
	QString name() const { return QLatin1String("KSqueezedTextLabel"); }
	QString toolTip() const { return QLatin1String("A QLabel that squeezes its text (KDE)"); }
	QString whatsThis() const { return QLatin1String("If the text is too long to fit into the label it is divided into remaining left and right parts which are separated by three dots"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KSqueezedTextLabel("KSqueezedTextLabel", parent);
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KImageFilePreviewPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KImageFilePreviewPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KImageFilePreviewPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kimagefilepreview.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Display (KDE)"); }
	QString includeFile() const { return QLatin1String("kimagefilepreview.h"); }
	QString name() const { return QLatin1String("KImageFilePreview"); }
	QString toolTip() const { return QLatin1String("A widget that provides a image preview of a URL"); }
	QString whatsThis() const { return QLatin1String("KImageFilePreview Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KImageFilePreview(parent);
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KRatingWidgetPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KRatingWidgetPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KRatingWidgetPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kratingwidget.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Display (KDE)"); }
	QString includeFile() const { return QLatin1String("kratingwidget.h"); }
	QString name() const { return QLatin1String("KRatingWidget"); }
	QString toolTip() const { return QLatin1String("Rating value display (KDE)"); }
	QString whatsThis() const { return QLatin1String("Displays a rating as a range of stars or other arbitrary pixmaps."); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KRatingWidget( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KAnimatedButtonPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KAnimatedButtonPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KAnimatedButtonPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kanimatedbutton.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Display (KDE)"); }
	QString includeFile() const { return QLatin1String("kanimatedbutton.h"); }
	QString name() const { return QLatin1String("KAnimatedButton"); }
	QString toolTip() const { return QLatin1String("An extended version of QToolButton which can display an animated icon."); }
	QString whatsThis() const { return QLatin1String("KAnimatedButton Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KAnimatedButton(parent);
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KColorCellsPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KColorCellsPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KColorCellsPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kcolorcells.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Input (KDE)"); }
	QString includeFile() const { return QLatin1String("kcolordialog.h"); }
	QString name() const { return QLatin1String("KColorCells"); }
	QString toolTip() const { return QLatin1String("Color Table (KDE)"); }
	QString whatsThis() const { return QLatin1String("KColorCells Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KColorCells(parent, 3, 5);
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KTreeWidgetSearchLineWidgetPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KTreeWidgetSearchLineWidgetPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KTreeWidgetSearchLineWidgetPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/ktreewidgetsearchlinewidget.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Input (KDE)"); }
	QString includeFile() const { return QLatin1String("ktreewidgetsearchline.h"); }
	QString name() const { return QLatin1String("KTreeWidgetSearchLineWidget"); }
	QString toolTip() const { return QLatin1String("QTreeWidget Search Line Widget (KDE)"); }
	QString whatsThis() const { return QLatin1String("KTreeWidgetSearchLineWidget Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KTreeWidgetSearchLineWidget(parent, 0);
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KShortcutsEditorPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KShortcutsEditorPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KShortcutsEditorPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kshortcutseditor.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Input (KDE)"); }
	QString includeFile() const { return QLatin1String("kshortcutseditor.h"); }
	QString name() const { return QLatin1String("KShortcutsEditor"); }
	QString toolTip() const { return QLatin1String("Shortcuts editor widget. (KDE)"); }
	QString whatsThis() const { return QLatin1String("A widget to edit a list of shortcuts."); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KShortcutsEditor( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KColorPatchPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KColorPatchPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KColorPatchPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kcolorpatch.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Display (KDE)"); }
	QString includeFile() const { return QLatin1String("kcolordialog.h"); }
	QString name() const { return QLatin1String("KColorPatch"); }
	QString toolTip() const { return QLatin1String("The KColorPatch widget is a (usually small) widget showing a selected color."); }
	QString whatsThis() const { return QLatin1String("KColorPatch Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KColorPatch(parent);
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class SonnetDictionaryComboBoxPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	SonnetDictionaryComboBoxPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~SonnetDictionaryComboBoxPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/sonnetdictionarycombobox.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Sonnet (KDE)"); }
	QString includeFile() const { return QLatin1String("sonnet/dictionarycombobox.h"); }
	QString name() const { return QLatin1String("Sonnet::DictionaryComboBox"); }
	QString toolTip() const { return QLatin1String("Dictionary Combobox"); }
	QString whatsThis() const { return QLatin1String("A combobox to select a dictionary for spellchecking"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new Sonnet::DictionaryComboBox( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KActionSelectorPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KActionSelectorPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KActionSelectorPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/kactionselector.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Views (KDE)"); }
	QString includeFile() const { return QLatin1String("kactionselector.h"); }
	QString name() const { return QLatin1String("KActionSelector"); }
	QString toolTip() const { return QLatin1String("A widget for selecting and arranging actions/objects"); }
	QString whatsThis() const { return QLatin1String("KActionSelector Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KActionSelector( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KHistoryComboBoxPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KHistoryComboBoxPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KHistoryComboBoxPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/khistorycombobox.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Input (KDE)"); }
	QString includeFile() const { return QLatin1String("khistorycombobox.h"); }
	QString name() const { return QLatin1String("KHistoryComboBox"); }
	QString toolTip() const { return QLatin1String("A combobox for offering a history and completion (KDE)"); }
	QString whatsThis() const { return QLatin1String("A combobox which implements a history like a unix shell"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KHistoryComboBox( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KDEWidgets : public QObject, public QDesignerCustomWidgetCollectionInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetCollectionInterface)
public:
	KDEWidgets(QObject *parent = 0);
	virtual ~KDEWidgets() {}
	QList<QDesignerCustomWidgetInterface*> customWidgets() const { return m_plugins; } 
	
private:
	QList<QDesignerCustomWidgetInterface*> m_plugins;
};

Q_EXPORT_PLUGIN2(KDEWidgets, KDEWidgets)

KDEWidgets::KDEWidgets(QObject *parent)
	: QObject(parent){
	(void) new KComponentData("makekdewidgets");
		m_plugins.append( new KDialogPlugin(this) );
		m_plugins.append( new KEditListWidgetPlugin(this) );
		m_plugins.append( new KPixmapRegionSelectorWidgetPlugin(this) );
		m_plugins.append( new KLanguageButtonPlugin(this) );
		m_plugins.append( new KIconButtonPlugin(this) );
		m_plugins.append( new KTextBrowserPlugin(this) );
		m_plugins.append( new KRichTextEditPlugin(this) );
		m_plugins.append( new KButtonGroupPlugin(this) );
		m_plugins.append( new KGradientSelectorPlugin(this) );
		m_plugins.append( new KDateTablePlugin(this) );
		m_plugins.append( new KColorComboPlugin(this) );
		m_plugins.append( new KDoubleNumInputPlugin(this) );
		m_plugins.append( new KKeySequenceWidgetPlugin(this) );
		m_plugins.append( new KCapacityBarPlugin(this) );
		m_plugins.append( new KTimeComboBoxPlugin(this) );
		m_plugins.append( new KFilterProxySearchLinePlugin(this) );
		m_plugins.append( new KFontComboBoxPlugin(this) );
		m_plugins.append( new KDateComboBoxPlugin(this) );
		m_plugins.append( new KColorButtonPlugin(this) );
		m_plugins.append( new KPageViewPlugin(this) );
		m_plugins.append( new KFileMetaDataWidgetPlugin(this) );
		m_plugins.append( new KTabBarPlugin(this) );
		m_plugins.append( new KFontRequesterPlugin(this) );
		m_plugins.append( new KDialogButtonBoxPlugin(this) );
		m_plugins.append( new KDateTimeEditPlugin(this) );
		m_plugins.append( new KFontChooserPlugin(this) );
		m_plugins.append( new KDatePickerPlugin(this) );
		m_plugins.append( new KComboBoxPlugin(this) );
		m_plugins.append( new KListWidgetSearchLinePlugin(this) );
		m_plugins.append( new KUrlComboRequesterPlugin(this) );
		m_plugins.append( new KDateWidgetPlugin(this) );
		m_plugins.append( new KCategorizedViewPlugin(this) );
		m_plugins.append( new KLineEditPlugin(this) );
		m_plugins.append( new KSeparatorPlugin(this) );
		m_plugins.append( new KCharSelectPlugin(this) );
		m_plugins.append( new KUrlLabelPlugin(this) );
		m_plugins.append( new KColorValueSelectorPlugin(this) );
		m_plugins.append( new KPushButtonPlugin(this) );
		m_plugins.append( new KLedPlugin(this) );
		m_plugins.append( new KPlotWidgetPlugin(this) );
		m_plugins.append( new KRestrictedLinePlugin(this) );
		m_plugins.append( new KIntSpinBoxPlugin(this) );
		m_plugins.append( new KTreeWidgetSearchLinePlugin(this) );
		m_plugins.append( new KTitleWidgetPlugin(this) );
		m_plugins.append( new KPageWidgetPlugin(this) );
		m_plugins.append( new KTextEditPlugin(this) );
		m_plugins.append( new KListWidgetPlugin(this) );
		m_plugins.append( new KRichTextWidgetPlugin(this) );
		m_plugins.append( new KPixmapSequenceWidgetPlugin(this) );
		m_plugins.append( new KTimeZoneWidgetPlugin(this) );
		m_plugins.append( new KMultiTabBarPlugin(this) );
		m_plugins.append( new KUrlRequesterPlugin(this) );
		m_plugins.append( new KIntNumInputPlugin(this) );
		m_plugins.append( new KShortcutWidgetPlugin(this) );
		m_plugins.append( new KTabWidgetPlugin(this) );
		m_plugins.append( new KDateTimeWidgetPlugin(this) );
		m_plugins.append( new KRulerPlugin(this) );
		m_plugins.append( new KSqueezedTextLabelPlugin(this) );
		m_plugins.append( new KImageFilePreviewPlugin(this) );
		m_plugins.append( new KRatingWidgetPlugin(this) );
		m_plugins.append( new KAnimatedButtonPlugin(this) );
		m_plugins.append( new KColorCellsPlugin(this) );
		m_plugins.append( new KTreeWidgetSearchLineWidgetPlugin(this) );
		m_plugins.append( new KShortcutsEditorPlugin(this) );
		m_plugins.append( new KColorPatchPlugin(this) );
		m_plugins.append( new SonnetDictionaryComboBoxPlugin(this) );
		m_plugins.append( new KActionSelectorPlugin(this) );
		m_plugins.append( new KHistoryComboBoxPlugin(this) );

}

#include </usr/src/ports/kdelibs/kdelibs-4.14.38-2.x86_64/build/kdewidgets/kdewidgets.moc>

