/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp -m -p avahi_domainbrowser_interface /usr/src/ports/kdelibs/kdelibs-4.14.38-2.x86_64/src/kdelibs-4.14.38/dnssd/org.freedesktop.Avahi.DomainBrowser.xml
 *
 * qdbusxml2cpp is Copyright (C) 2015 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#include "avahi_domainbrowser_interface.h"

/*
 * Implementation of interface class OrgFreedesktopAvahiDomainBrowserInterface
 */

OrgFreedesktopAvahiDomainBrowserInterface::OrgFreedesktopAvahiDomainBrowserInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent)
    : QDBusAbstractInterface(service, path, staticInterfaceName(), connection, parent)
{
}

OrgFreedesktopAvahiDomainBrowserInterface::~OrgFreedesktopAvahiDomainBrowserInterface()
{
}


#include "avahi_domainbrowser_interface.moc"
