/****************************************************************************
** Meta object code from reading C++ file 'khtmlfind_p.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.7)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/kdelibs-4.14.38/khtml/ui/findbar/khtmlfind_p.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'khtmlfind_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.7. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_KHTMLFind[] = {

 // content:
       6,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: signature, parameters, type, tag, flags
      28,   11,   10,   10, 0x05,

 // slots: signature, parameters, type, tag, flags
      59,   10,   10,   10, 0x0a,
      70,   10,   10,   10, 0x0a,
      83,   10,   10,   10, 0x08,
     103,   10,   10,   10, 0x08,
     140,  126,   10,   10, 0x08,
     171,   10,   10,   10, 0x08,
     191,   10,   10,   10, 0x08,
     206,   10,   10,   10, 0x08,

       0        // eod
};

static const char qt_meta_stringdata_KHTMLFind[] = {
    "KHTMLFind\0\0selection,length\0"
    "foundMatch(DOM::Selection,int)\0"
    "activate()\0deactivate()\0slotFindDestroyed()\0"
    "slotSelectionChanged()\0,index,length\0"
    "slotHighlight(QString,int,int)\0"
    "slotSearchChanged()\0slotFindNext()\0"
    "slotFindPrevious()\0"
};

void KHTMLFind::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        KHTMLFind *_t = static_cast<KHTMLFind *>(_o);
        switch (_id) {
        case 0: _t->foundMatch((*reinterpret_cast< const DOM::Selection(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 1: _t->activate(); break;
        case 2: _t->deactivate(); break;
        case 3: _t->slotFindDestroyed(); break;
        case 4: _t->slotSelectionChanged(); break;
        case 5: _t->slotHighlight((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 6: _t->slotSearchChanged(); break;
        case 7: _t->slotFindNext(); break;
        case 8: _t->slotFindPrevious(); break;
        default: ;
        }
    }
}

const QMetaObjectExtraData KHTMLFind::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject KHTMLFind::staticMetaObject = {
    { &QObject::staticMetaObject, qt_meta_stringdata_KHTMLFind,
      qt_meta_data_KHTMLFind, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &KHTMLFind::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *KHTMLFind::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *KHTMLFind::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_KHTMLFind))
        return static_cast<void*>(const_cast< KHTMLFind*>(this));
    return QObject::qt_metacast(_clname);
}

int KHTMLFind::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void KHTMLFind::foundMatch(const DOM::Selection & _t1, int _t2)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_END_MOC_NAMESPACE
