/****************************************************************************
** Meta object code from reading C++ file 'kselectionproxymodel.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.7)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/kdelibs-4.14.38/kdeui/itemviews/kselectionproxymodel.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kselectionproxymodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.7. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_KSelectionProxyModel[] = {

 // content:
       6,       // revision
       0,       // classname
       0,    0, // classinfo
      17,   14, // methods
       0,    0, // properties
       1,   99, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: signature, parameters, type, tag, flags
      38,   22,   21,   21, 0x05,
      86,   77,   21,   21, 0x05,
     124,  114,   21,   21, 0x05,
     170,  114,   21,   21, 0x05,

 // slots: signature, parameters, type, tag, flags
     208,  205,   21,   21, 0x08,
     257,  205,   21,   21, 0x08,
     297,  205,   21,   21, 0x08,
     345,  205,   21,   21, 0x08,
     389,  384,   21,   21, 0x08,
     451,  384,   21,   21, 0x08,
     504,   21,   21,   21, 0x08,
     532,   21,   21,   21, 0x08,
     551,   21,   21,   21, 0x08,
     582,   21,   21,   21, 0x08,
     606,  604,   21,   21, 0x08,
     669,  649,   21,   21, 0x08,
     717,   21,   21,   21, 0x08,

 // enums: name, flags, count, data
     740, 0x0,    5,  103,

 // enum data: key, value
     755, uint(KSelectionProxyModel::SubTrees),
     764, uint(KSelectionProxyModel::SubTreeRoots),
     777, uint(KSelectionProxyModel::SubTreesWithoutRoots),
     798, uint(KSelectionProxyModel::ExactSelection),
     813, uint(KSelectionProxyModel::ChildrenOfExactSelection),

       0        // eod
};

static const char qt_meta_stringdata_KSelectionProxyModel[] = {
    "KSelectionProxyModel\0\0removeRootIndex\0"
    "rootIndexAboutToBeRemoved(QModelIndex)\0"
    "newIndex\0rootIndexAdded(QModelIndex)\0"
    "selection\0rootSelectionAboutToBeRemoved(QItemSelection)\0"
    "rootSelectionAdded(QItemSelection)\0"
    ",,\0sourceRowsAboutToBeInserted(QModelIndex,int,int)\0"
    "sourceRowsInserted(QModelIndex,int,int)\0"
    "sourceRowsAboutToBeRemoved(QModelIndex,int,int)\0"
    "sourceRowsRemoved(QModelIndex,int,int)\0"
    ",,,,\0sourceRowsAboutToBeMoved(QModelIndex,int,int,QModelIndex,int)\0"
    "sourceRowsMoved(QModelIndex,int,int,QModelIndex,int)\0"
    "sourceModelAboutToBeReset()\0"
    "sourceModelReset()\0sourceLayoutAboutToBeChanged()\0"
    "sourceLayoutChanged()\0,\0"
    "sourceDataChanged(QModelIndex,QModelIndex)\0"
    "selected,deselected\0"
    "selectionChanged(QItemSelection,QItemSelection)\0"
    "sourceModelDestroyed()\0FilterBehavior\0"
    "SubTrees\0SubTreeRoots\0SubTreesWithoutRoots\0"
    "ExactSelection\0ChildrenOfExactSelection\0"
};

void KSelectionProxyModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        KSelectionProxyModel *_t = static_cast<KSelectionProxyModel *>(_o);
        switch (_id) {
        case 0: _t->rootIndexAboutToBeRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 1: _t->rootIndexAdded((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 2: _t->rootSelectionAboutToBeRemoved((*reinterpret_cast< const QItemSelection(*)>(_a[1]))); break;
        case 3: _t->rootSelectionAdded((*reinterpret_cast< const QItemSelection(*)>(_a[1]))); break;
        case 4: _t->d_func()->sourceRowsAboutToBeInserted((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 5: _t->d_func()->sourceRowsInserted((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 6: _t->d_func()->sourceRowsAboutToBeRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 7: _t->d_func()->sourceRowsRemoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 8: _t->d_func()->sourceRowsAboutToBeMoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< const QModelIndex(*)>(_a[4])),(*reinterpret_cast< int(*)>(_a[5]))); break;
        case 9: _t->d_func()->sourceRowsMoved((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< const QModelIndex(*)>(_a[4])),(*reinterpret_cast< int(*)>(_a[5]))); break;
        case 10: _t->d_func()->sourceModelAboutToBeReset(); break;
        case 11: _t->d_func()->sourceModelReset(); break;
        case 12: _t->d_func()->sourceLayoutAboutToBeChanged(); break;
        case 13: _t->d_func()->sourceLayoutChanged(); break;
        case 14: _t->d_func()->sourceDataChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2]))); break;
        case 15: _t->d_func()->selectionChanged((*reinterpret_cast< const QItemSelection(*)>(_a[1])),(*reinterpret_cast< const QItemSelection(*)>(_a[2]))); break;
        case 16: _t->d_func()->sourceModelDestroyed(); break;
        default: ;
        }
    }
}

const QMetaObjectExtraData KSelectionProxyModel::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject KSelectionProxyModel::staticMetaObject = {
    { &QAbstractProxyModel::staticMetaObject, qt_meta_stringdata_KSelectionProxyModel,
      qt_meta_data_KSelectionProxyModel, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &KSelectionProxyModel::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *KSelectionProxyModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *KSelectionProxyModel::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_KSelectionProxyModel))
        return static_cast<void*>(const_cast< KSelectionProxyModel*>(this));
    return QAbstractProxyModel::qt_metacast(_clname);
}

int KSelectionProxyModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractProxyModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void KSelectionProxyModel::rootIndexAboutToBeRemoved(const QModelIndex & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void KSelectionProxyModel::rootIndexAdded(const QModelIndex & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void KSelectionProxyModel::rootSelectionAboutToBeRemoved(const QItemSelection & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void KSelectionProxyModel::rootSelectionAdded(const QItemSelection & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
QT_END_MOC_NAMESPACE
