/**
* This file was autogenerated by makekdewidgets. Any changes will be lost!
* The generated code in this file is licensed under the same license that the
* input file.
*/
#include <QIcon>
#include <QtDesigner/QDesignerContainerExtension>
#include <QDesignerCustomWidgetInterface>
#include <qplugin.h>
#include <qdebug.h>

#include <kcomponentdata.h>
#include <classpreviews.h>
#include <ktimezone.h>
#include <k3listview.h>
#include <k3iconview.h>


class K3ListViewPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	K3ListViewPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~K3ListViewPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/k3listview.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Views (KDE)"); }
	QString includeFile() const { return QLatin1String("k3listview.h"); }
	QString name() const { return QLatin1String("K3ListView"); }
	QString toolTip() const { return QLatin1String("Extended List View (KDE)"); }
	QString whatsThis() const { return QLatin1String("An improved version of the Q3ListView that allows certain KDE extensions"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new K3ListView( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class K3IconViewPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	K3IconViewPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~K3IconViewPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/k3iconview.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Views (KDE)"); }
	QString includeFile() const { return QLatin1String("k3iconview.h"); }
	QString name() const { return QLatin1String("K3IconView"); }
	QString toolTip() const { return QLatin1String("Extended Icon View (KDE)"); }
	QString whatsThis() const { return QLatin1String("An improved version of Q3IconView that allows certain KDE extensions"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new K3IconView( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KDE3SupportWidgets : public QObject, public QDesignerCustomWidgetCollectionInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetCollectionInterface)
public:
	KDE3SupportWidgets(QObject *parent = 0);
	virtual ~KDE3SupportWidgets() {}
	QList<QDesignerCustomWidgetInterface*> customWidgets() const { return m_plugins; } 
	
private:
	QList<QDesignerCustomWidgetInterface*> m_plugins;
};

Q_EXPORT_PLUGIN2(KDE3SupportWidgets, KDE3SupportWidgets)

KDE3SupportWidgets::KDE3SupportWidgets(QObject *parent)
	: QObject(parent){
	(void) new KComponentData("makekdewidgets");
		m_plugins.append( new K3ListViewPlugin(this) );
		m_plugins.append( new K3IconViewPlugin(this) );

}

#include </usr/src/ports/kdelibs/kdelibs-4.14.38-2.x86_64/build/kdewidgets/kde3supportwidgets.moc>

