/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp -m -a kimifaceadaptor -i kimproxy.h -l KIMProxy /usr/src/ports/kdelibs/kdelibs-4.14.38-2.x86_64/src/kdelibs-4.14.38/interfaces/kimproxy/interface/org.kde.KIMIface.xml
 *
 * qdbusxml2cpp is Copyright (C) 2015 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "kimifaceadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class KIMAdaptor
 */

KIMAdaptor::KIMAdaptor(KIMProxy *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

KIMAdaptor::~KIMAdaptor()
{
    // destructor
}

bool KIMAdaptor::addContact(const QString &contactId, const QString &protocol)
{
    // handle method call org.kde.KIM.addContact
    return parent()->addContact(contactId, protocol);
}

QStringList KIMAdaptor::allContacts()
{
    // handle method call org.kde.KIM.allContacts
    return parent()->allContacts();
}

bool KIMAdaptor::canReceiveFiles(const QString &uid)
{
    // handle method call org.kde.KIM.canReceiveFiles
    return parent()->canReceiveFiles(uid);
}

bool KIMAdaptor::canRespond(const QString &uid)
{
    // handle method call org.kde.KIM.canRespond
    return parent()->canRespond(uid);
}

void KIMAdaptor::chatWithContact(const QString &uid)
{
    // handle method call org.kde.KIM.chatWithContact
    parent()->chatWithContact(uid);
}

QString KIMAdaptor::context(const QString &uid)
{
    // handle method call org.kde.KIM.context
    return parent()->context(uid);
}

QString KIMAdaptor::displayName(const QString &uid)
{
    // handle method call org.kde.KIM.displayName
    return parent()->displayName(uid);
}

QStringList KIMAdaptor::fileTransferContacts()
{
    // handle method call org.kde.KIM.fileTransferContacts
    return parent()->fileTransferContacts();
}

bool KIMAdaptor::isPresent(const QString &uid)
{
    // handle method call org.kde.KIM.isPresent
    return parent()->isPresent(uid);
}

QString KIMAdaptor::locate(const QString &contactId, const QString &protocol)
{
    // handle method call org.kde.KIM.locate
    return parent()->locate(contactId, protocol);
}

void KIMAdaptor::messageContact(const QString &uid, const QString &message)
{
    // handle method call org.kde.KIM.messageContact
    parent()->messageContact(uid, message);
}

QStringList KIMAdaptor::onlineContacts()
{
    // handle method call org.kde.KIM.onlineContacts
    return parent()->onlineContacts();
}

QString KIMAdaptor::presenceString(const QString &uid)
{
    // handle method call org.kde.KIM.presenceString
    return parent()->presenceString(uid);
}

QStringList KIMAdaptor::reachableContacts()
{
    // handle method call org.kde.KIM.reachableContacts
    return parent()->reachableContacts();
}

void KIMAdaptor::sendFile(const QString &uid, const QString &sourceURL)
{
    // handle method call org.kde.KIM.sendFile
    parent()->sendFile(uid, sourceURL);
}

void KIMAdaptor::sendFile(const QString &uid, const QString &sourceURL, const QString &altFileName)
{
    // handle method call org.kde.KIM.sendFile
    parent()->sendFile(uid, sourceURL, altFileName);
}

void KIMAdaptor::sendFile(const QString &uid, const QString &sourceURL, const QString &altFileName, qlonglong fileSize)
{
    // handle method call org.kde.KIM.sendFile
    parent()->sendFile(uid, sourceURL, altFileName, fileSize);
}


#include "kimifaceadaptor.moc"
